#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using System;
using System.Drawing;

namespace HeuristicLab.Optimization.Networks {
  [Item("OutputPort", "An asynchronous output port of an optimization network node.")]
  [StorableClass]
  public class OutputPort<T> : ValuePort<T>, IOutputPort<T> where T : class, IItem {
    public static new Image StaticItemImage {
      get { return HeuristicLab.Common.Resources.VSImageLibrary.ArrowUp; }
    }

    new public T Value {
      get { return base.Value; }
      set { base.Value = value; }
    }
    IItem IOutputPort.Value {
      get { return Value; }
      set {
        T val = value as T;
        if ((value != null) && (val == null))
          throw new InvalidOperationException(
            string.Format("Type mismatch. Value is not a \"{0}\".",
                          typeof(T).GetPrettyName())
          );
        Value = val;
      }
    }

    [StorableConstructor]
    protected OutputPort(bool deserializing) : base(deserializing) { }
    protected OutputPort(OutputPort<T> original, Cloner cloner) : base(original, cloner) { }
    public OutputPort() : base("OutputPort") { }
    public OutputPort(string name) : base(name) { }
    public OutputPort(string name, string description) : base(name, description) { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new OutputPort<T>(this, cloner);
    }
  }
}
